#include <algorithm>
#include "builder.h"
#include "file.h"
#include "sourcearchivecomponent.h"
#include "sourcepackage.h"
#include "tool.h"

using namespace std;

SourceArchiveComponent::SourceArchiveComponent(SourcePackage &p):
	Component(p, p.get_name()+"-source")
{ }

void SourceArchiveComponent::create_targets() const
{
	Builder &builder = package.get_builder();

	list<Target *> files;
	files.insert(files.begin(), &package.get_build_file());

	SourceList source_filenames = collect_source_files();
	for(SourceList::const_iterator i=source_filenames.begin(); i!=source_filenames.end(); ++i)
	{
		FileTarget *file = builder.get_vfs().get_target(*i);
		if(!file)
			file = new File(builder, package, *i);
		files.push_back(file);
	}

	BuildGraph &build_graph = builder.get_build_graph();
	const BuildGraph::TargetMap &targets = build_graph.get_targets();
	for(BuildGraph::TargetMap::const_iterator i=targets.begin(); i!=targets.end(); ++i)
		if(i->second->get_package()==&package && !i->second->is_buildable())
			if(find(files.begin(), files.end(), i->second)==files.end())
				files.push_back(i->second);

	const Toolchain &toolchain = builder.get_toolchain();
	string archive_name = package.get_name();
	if(!package.get_version().empty())
		archive_name += "-"+package.get_version();
	archive_name += "-source";
	Target *result = toolchain.get_tool("TAR").create_target(files, archive_name);
	build_graph.get_target("archives")->add_dependency(*result);
}
